/* Dave Pallot */
/* pallot@iinet.net.au */
/* Copyright 2000 */

#include "Socket.h"

bool Socket::m_init = false;
unsigned int Socket::m_instance = 0;

Socket::Socket() :
	m_socket(INVALID_SOCKET),
	m_state(Closed)
{
	if (m_init == false)
	{
		WSADATA socketData;

		int retCode = ::WSAStartup(0x0101, &socketData);

		if (retCode == 0)
		{
			m_init = true;
		}
	}

	m_instance++;
}


Socket::~Socket()
{
	close();
	
	if (m_init == true)
	{
		m_instance--;

		if (m_instance == 0)
		{
			::WSACleanup();
		}
	}
}


int Socket::initSocket()
{
	if (m_socket == INVALID_SOCKET)
	{
		m_socket = ::socket(AF_INET, SOCK_STREAM, 0);

		if (m_socket != INVALID_SOCKET)
		{
			m_state = Init;
			
			// Add debug info
			#ifdef _DEBUG
				int opt = SO_DEBUG;
				::setsockopt(m_socket, SOL_SOCKET, 0, (char *)opt, sizeof(opt));
			#endif	
			
		}
	}

	return m_socket;
}


void Socket::close()
{
	if (m_socket != INVALID_SOCKET)
	{
		if (::closesocket(m_socket) != SOCKET_ERROR)
		{
			m_state = Closed;
			m_socket = INVALID_SOCKET;
		}
	}
}


int Socket::connect(const char* hostname, unsigned short port)
{
	int retCode = SOCKET_ERROR;
	
	if (m_socket != INVALID_SOCKET && m_state == Init)
	{
		// Arrives in network host order
		struct hostent* host = ::gethostbyname(hostname);

		if (host != NULL)
		{
			unsigned long ipAddress = 0;

			::memcpy( (char*) &(ipAddress), host->h_addr, host->h_length );

			m_inetAddr.m_address.sin_family = AF_INET;				   // Socket Type
			m_inetAddr.m_address.sin_addr.S_un.S_addr = ipAddress;   // IP Address	
			m_inetAddr.m_address.sin_port = ::htons(port);			   // Port

			retCode = ::connect(m_socket, (struct sockaddr*)&m_inetAddr.m_address, sizeof(struct sockaddr));
		
			if (retCode == 0)
			{
				m_state = Connected;
			}
		}
	}

	return retCode;
}


int Socket::listen(unsigned short port, int backlog)
{
	int retCode = SOCKET_ERROR;

	if (m_socket != INVALID_SOCKET && m_state == Init)
	{
		m_inetAddr.m_address.sin_family = AF_INET;
		m_inetAddr.m_address.sin_addr.S_un.S_addr = ::htonl(INADDR_ANY); // Listen on Any Address
		m_inetAddr.m_address.sin_port = ::htons(port);

		retCode = ::bind(m_socket, (struct sockaddr*)&m_inetAddr.m_address, sizeof(m_inetAddr.m_address));
		
		if (retCode == 0)
		{
			retCode = ::listen(m_socket, backlog);

			if (retCode == 0)
			{	
				m_state = Listening;
			}
		}
	}

	return retCode;
}




int Socket::accept(Socket& socket)
{
	// Close accepting socket
	socket.close();
	
	// We must be listening before we can accept
	if (m_state == Listening)
	{
		int length = sizeof(struct sockaddr);

		socket.m_socket = ::accept(m_socket, (struct sockaddr*)&socket.m_inetAddr.m_address, &length);
		
		if (socket.m_socket != INVALID_SOCKET)
		{
			socket.m_state = Connected;
		}
		else
		{
			// Didn't accept client so close it down
			socket.close();
		}
	}

	return socket.m_socket;
}


unsigned int Socket::sendBuffer(const char* buffer, unsigned int size)
{
	int bytesSent = 0;

	if (m_state == Connected)
	{
		bytesSent = ::send(m_socket, buffer, size, 0);

		if (bytesSent == 0 || bytesSent == SOCKET_ERROR)
		{
			close();
		}
	}

	return bytesSent;
}


unsigned int Socket::readBuffer(char* buffer, unsigned int size)
{
	int bytesReadTotal = 0;
	char* ptr = buffer;

	if (m_state == Connected)
	{
		int bytesRead = 0;
		// Always good to make sure we have the whole packet
		while (size > 0)
		{
			bytesRead = ::recv(m_socket, ptr, size, 0);
			
			if (bytesRead == 0 || bytesRead == SOCKET_ERROR)
			{
				close();
				break;
			}
			
			size -= bytesRead;
			ptr += bytesRead;
			bytesReadTotal += bytesRead;
		}
	}

	return bytesReadTotal;
}


Socket& Socket::operator >> (bool& data)
{
	if (m_state == Connected)
	{
		int read = ::recv(m_socket, (char*)&data, sizeof(bool), 0);

		if (read == 0 || read == SOCKET_ERROR)
		{
			close();
		}
	}

	return *this;
}


Socket& Socket::operator >> (char& data)
{	
	if (m_state == Connected)
	{
		int read = ::recv(m_socket, &data, sizeof(char), 0);

		if (read == 0 || read == SOCKET_ERROR)
		{
			close();
		}
	}

	return *this;
}

Socket& Socket::operator >> (int& data)
{
	if (m_state == Connected)
	{
		int size = sizeof(int);
		int value = 0;

		int bytesRead = 0;

		while (size > 0)
		{
			bytesRead = ::recv(m_socket, (char*)&(value)+bytesRead, size, 0);

			if (bytesRead == 0 || bytesRead == SOCKET_ERROR)
			{
				close();
				break;
			}
			
			size -= bytesRead;
		}

		if (bytesRead > 0)
		{	
			data = ::ntohl(value);
		}

	}

	return *this;
}


Socket& Socket::operator >> (unsigned int& data)
{
	if (m_state == Connected)
	{
		int size = sizeof(unsigned int);
		unsigned int value = 0;

		int bytesRead = 0;

		while (size > 0)
		{
			bytesRead = ::recv(m_socket, (char*)&(value)+bytesRead, size, 0);

			if (bytesRead == 0 || bytesRead == SOCKET_ERROR)
			{
				close();
				break;
			}
			
			size -= bytesRead;
		}

		if (bytesRead > 0)
		{
			data = ::ntohl(value);
		}

	}

	return *this;
}

Socket& Socket::operator << (bool data)
{
	if (m_state == Connected)
	{
		int sent = ::send(m_socket, (char*)&data, sizeof(bool), 0);

		if (sent == 0 || sent == SOCKET_ERROR)
		{
			close();
		}
	}

	return *this;
}


Socket& Socket::operator << (char data)
{
	if (m_state == Connected)
	{
		int sent = ::send(m_socket, (char*)&data, sizeof(char), 0);

		if (sent == 0 || sent == SOCKET_ERROR)
		{
			close();
		}
	}

	return *this;
}

Socket& Socket::operator << (int data)
{
	if (m_state == Connected)
	{
		int tmpData = ::htonl(data);

		int sent = ::send(m_socket, (char*)&tmpData, sizeof(int), 0);

		if (sent == 0 || sent == SOCKET_ERROR)
		{
			close();
		}
	}

	return *this;
}


Socket& Socket::operator << (unsigned int data)
{
	if (m_state == Connected)
	{
		int tmpData = ::htonl(data);

		int sent = ::send(m_socket, (char*)&tmpData, sizeof(unsigned int), 0);

		if (sent == 0 || sent == SOCKET_ERROR)
		{
			close();
		}
	}

	return *this;
}

